@@ -331,7 +331,7 @@ class WSGIHandlerMixin:
 # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/442473
 #
 try:
-    from OpenSSL import SSL, tsafe
+    from OpenSSL import SSL
     SocketErrors = (socket.error, SSL.ZeroReturnError, SSL.SysCallError)
 except ImportError:
     # Do not require pyOpenSSL to be installed, but disable SSL
@@ -379,7 +379,7 @@ else:
                                         self.socket_type)
             self.ssl_context = ssl_context
             if ssl_context:
-                class TSafeConnection(tsafe.Connection):
+                class SSLConnection(SSL.Connection):
                     def settimeout(self, *args):
                         self._lock.acquire()
                         try:
@@ -392,7 +392,7 @@ else:
                             return self._ssl_conn.gettimeout()
                         finally:
                             self._lock.release()
-                self.socket = TSafeConnection(ssl_context, self.socket)
+                self.socket = SSLConnection(ssl_context, self.socket)
             self.server_bind()
             if request_queue_size:
                 self.socket.listen(request_queue_size)
@@ -513,6 +513,7 @@ class LimitedLengthFile(object):
         if self.length - self._consumed <= 0:
             raise StopIteration
         return self.readline()
+    __next__ = next
 
     ## Optional methods ##
 
@@ -734,7 +735,11 @@ class ThreadPool(object):
             raise RuntimeError(
                 "Cannot kill worker; killthread/ctypes not available")
         thread_obj = threading._active.get(thread_id)
-        killthread.async_raise(thread_id, SystemExit)
+        try:
+            killthread.async_raise(thread_id, SystemExit)
+        except ValueError:
+            # invalid thread id -- the thread has died in the mean time
+            pass
         try:
             del self.worker_tracker[thread_id]
         except KeyError:
@@ -1350,7 +1355,7 @@ def server_runner(wsgi_app, global_conf, **kwargs):
         if name in kwargs:
             kwargs[name] = asbool(kwargs[name])
     threadpool_options = {}
-    for name, value in kwargs.items():
+    for name, value in list(kwargs.items()):
         if name.startswith('threadpool_') and name != 'threadpool_workers':
             threadpool_options[name[len('threadpool_'):]] = value
             del kwargs[name]
